/**
 * File:  lesson4_ex2c_main.c
 *
 * This program shows how to use the AVR Timer/Counter TC0 (8 bits)
 * to generate a pulse waveform with variable duty cycle.
 * The PWM output signal may be used to drive a LED with variable brightness.
 *
 * Hardware platform:  ATmega328P 'X-mini' kit (or AVR^BED)
 * Code library reqd:  lib_avrXmini (or lib_avrbed)
 *
 * Reference:  AVR C Tutorial, Lesson 4
 * ``````````````````````````````````````
 * X-mini board wiring information:
 * ``````````````````````````````````
 *    Pin PD6/OC0A is wired to a LED to be driven by the PWM output.
 *    Pin PC3/ADC3 is wired to the potentiometer arm (for ADC input)
 *    See "lib_avrXmini.h" for LCD panel signal connections.
 *
 * Author:  Mike Bauer   2020-10-14  (v2.1 for AVR X-mini)
 */
#include <avr/io.h>
#define F_CPU  16000000UL     // or 8000000UL for 'AVR^BED' (8MHz)

// These header files contain definitions needed by library functions:
#include <stdlib.h>
#include <string.h>
#include "lib_avrXmini.h"  // or "lib_avrbed.h" for 'AVR^BED'

// Macros to enable & disable Timer-Counter TC0 Overflow (OVF) interrupt
#define TC0_OVF_IRQ_ENABLE()   (TIMSK0 |= (1<<TOIE0))
#define TC0_OVF_IRQ_DISABLE()  (TIMSK0 &= ~(1<<TOIE0))

// ------------  Global variables  ----------------------------
BYTE  PWM_duty_pc;                // PWM pulse duty, percent (0..100 %)
BYTE  last_PWM_duty;              // PWM pulse duty, last value set
int   adc_count;                  // Raw ADC conversion count (0..1023)
unsigned long  intervalEndTime;   // 200ms update interval end time (ms)
char  textbuf[16];                // text buffer for itoa() function

// ------------  Function prototype declarations  --------------
void  TC0_Setup();


int  main(void)
{
    PWM_duty_pc = 20;             // Set initial PWM pulse duty (20 %)
	last_PWM_duty = 255;
    
    // Initialize the LCD module and display header info on top line.
    lcd_initialise();
    lcd_command(LCD_CLR);
    lcd_command(LCD_CURSOR_OFF);
	lcd_cursor_posn(0, 0);
	lcd_print_string("ADC in  PWM duty");
    
    CLEAR_BIT(DDRC, 3);           // Make PC3 an input for the pot (ADC3)
    SET_BIT(DDRD, 6);             // make PD6 an output to drive a LED

    TC0_Setup();                  // TC0 is set up for PWM output
    TC1_initialize();             // Set up TC1 for library timer functions
    GLOBAL_INT_ENABLE();
	
	// Capture end-time for initial 200ms interval = NOW + 200 ms
	intervalEndTime = milliseconds() + 200;  

    while (TRUE)  // loop forever
    {
		if (milliseconds() >= intervalEndTime)  // 200ms interval expired...
		{
			// Capture end-time for next 200ms interval
			intervalEndTime = milliseconds() + 200;
			
			// Read the pot analogue input (ADC3), convert the reading to 
			// a percentage and set PWM duty (PWM_duty_pc) accordingly. 
			adc_count = ADC_ReadInput(3) - 5;   // compensate offset error
			if (adc_count < 0)  adc_count = 0;
			if (adc_count > 1000)  adc_count = 1000;
			PWM_duty_pc = (adc_count + 5) / 10;   // round to nearest %
			
			// Display the ADC reading and PWM duty (%) on the LCD:
			if (PWM_duty_pc != last_PWM_duty)   // PWM value has changed...
			{
				lcd_cursor_posn(1, 0);   
				lcd_print_string("                ");  // erase bottom line
				
				itoa(adc_count, textbuf, 10);   // display ADC count
				lcd_cursor_posn(1, 0);
				lcd_print_string(textbuf);
				
				itoa(PWM_duty_pc, textbuf, 10);   // display PWM duty
				lcd_cursor_posn(1, 8);
				lcd_print_string(textbuf);
				lcd_write_char('%');
				
				last_PWM_duty = PWM_duty_pc;  // update
			}
		}
    }
}


/*
 * Function:  TC0_Setup()
 *
 * This function initializes Timer-Counter TC0 in "Fast PWM" mode to generate a 
 * variable-duty pulse waveform on pin OC0A (= PD6).
 *
 * The timer "Output Compare" feature is used to make the timing automatic.
 * When the counter register (TMR0) reaches its "MAX count" (255), the count
 * register is automatically reset (zeroed) and an IRQ flag (TOV) is raised.
 * At the same time, the PWM output pin PD6/OC0A is set High (1).
 * 
 * When the counter register (TMR0) matches the Output Compare register value,
 * the PWM output signal on pin PD6/OC0A is reset LOW (0) automatically.
 * The PWM pulse duty is proportional to the OCR0A register value.
 *
 * The timer clock pre-scaler is set to divide the CPU clock frequency by 64.
 * If the CPU clock freq. is 16 MHz, the counter clock will be 250 kHz.
 * One complete counter cycle is 256 clocks, so the PWM output pulse frequency
 * will be (250kHz / 256) = 977 Hz (approx.)
 */
void  TC0_Setup()
{
    TCCR0A = 0b10000011;        // "Fast PWM" mode enabled on pin OC0A/PD6
    TCCR0B = 0x03;              // Prescaler = F_CPU / 64

    TC0_OVF_IRQ_ENABLE();       // Enable interrupt on Top count (OVerflow)
}


/*
 * Timer-Counter TC0 (8-bit TMR0 reg.) Interrupt Service Routine
 *
 * The timer will generate an IQR, hence this routine will be executed, when the
 * timer register (TMR0) reaches its MAX count value (255). The timer will reset
 * (TMR0 = 0) and the PWM output will be set HIGH on the next clock pulse.
 *
 * The PWM duty (set by register OCR0A) is updated here.
 * To avoid erratic PWM behaviour, OCR0A should not be written to anywhere else.
 */
ISR( TIMER0_OVF_vect )
{
    unsigned itemp = (255 * PWM_duty_pc) / 100;  // range 0..255
    
    OCR0A = LO_BYTE(itemp);   // copy lower 8 bits of itemp to OCR0A
}

// end of file

