/**
 * File:  lesson4_ex2c_outline.c
 *
 * This outline program shows how to use the AVR Timer/Counter TC0 (8 bits)
 * to generate a pulse waveform with variable duty cycle.
 * The PWM output signal may be used to drive a LED with variable brightness.
 *
 * Reference:  AVR C Tutorial, Lesson 4
 * ``````````````````````````````````````
 * X-mini board wiring information:
 * ``````````````````````````````````
 *    Pin PD6/OC0A is wired to a LED to be driven by the PWM output.
 *    Pin PC3/ADC3 is wired to the potentiometer arm. (for ADC input)
 *    See "lib_avrXmini.h" for LCD panel signal connections.
 *
 * Originated:  Mike Bauer    2020-10-14  (v2.1 for AVR X-mini)
 * Modified:    <your name>   20xx-xx-xx
 *
 */
#include <avr/io.h>
#define F_CPU  16000000UL     // X-mini CPU is clocked at 16 MHz

// These header files contain definitions needed by library functions:
#include <stdlib.h>
#include <string.h>
#include "lib_avrXmini.h"

// Macros to enable & disable Timer-Counter TC0 Overflow (OVF) interrupt
#define TC0_OVF_IRQ_ENABLE()   (TIMSK0 |= (1<<TOIE0))
#define TC0_OVF_IRQ_DISABLE()  (TIMSK0 &= ~(1<<TOIE0))


// ------------  Global variables  ----------------------------
unsigned char  PWM_duty_pc;        // PWM pulse duty, percent (0..100 %)
unsigned long  intervalEndTime;    // 200ms update interval end time (ms)


// ------------  Function prototype declarations  --------------
void  TC0_Setup();


int  main(void)
{
    PWM_duty_pc = 20;             // Set initial PWM pulse duty (20 %)
    
    // Add your code here to initialize the LCD module and show start-up text.
    ;
    ;
    ;
    
    CLEAR_BIT(DDRC, 3);           // Make PC3 an input for the pot (ADC)
    SET_BIT(DDRD, 6);             // make PD6 an output to drive a LED

    TC0_Setup();                  // TC0 is set up for PWM output
    TC1_initialize();             // Set up TC1 for milliseconds() function
    GLOBAL_INT_ENABLE();
    
    // Capture end-time for initial 200ms interval = NOW + 200 ms
    intervalEndTime = milliseconds() + 200;  

    while (TRUE)  // loop forever
    {
        if ( /* 200ms interval timer expired */ )
        {
            // Capture end-time for next 200ms interval
            ;
            
            // Add code below to read the pot analogue input (ADC3), convert the
            // reading to a percentage and set PWM duty (PWM_duty_pc) accordingly. 
            // This should be done at intervals of 200ms.
            ;
            ;
            ;
            
            // Display the PWM duty (%) on the LCD module.
            // The display should be updated every 200ms, but only when data changes.
            ;
            ;
            ;
        }
    }
}


// ----------------------------------------------------------------------------
// The functions below do not need modification, but you should understand how
// they work.  Refer to the AVR C Tutorial, Lesson 4.
// ----------------------------------------------------------------------------

/*
 * Function:  TC0_Setup()
 *
 * This function initializes Timer-Counter TC0 in "Fast PWM" mode to generate a 
 * variable-duty pulse waveform on pin OC0A (= PD6).
 *
 * The timer "Output Compare" feature is used to make the timing automatic.
 * When the counter register (TMR0) reaches its "MAX count" (255), the count
 * register is automatically reset (zeroed) and an IRQ flag (TOV) is raised.
 * At the same time, the PWM output pin PD6/OC0A is set High (1).
 * 
 * When the counter register (TMR0) matches the Output Compare register value,
 * the PWM output signal on pin PD6/OC0A is reset LOW (0) automatically.
 * The PWM pulse duty is proportional to the OCR0A register value.
 *
 * The timer clock pre-scaler is set to divide the CPU clock frequency by 64.
 * If the CPU clock freq. is 16 MHz, the counter clock will be 250 kHz.
 * One complete counter cycle is 256 clocks, so the PWM output pulse frequency
 * will be 250kHz / 256 = 977 Hz (approx.)
 */
void  TC0_Setup()
{
    TCCR0A = 0b10000011;        // "Fast PWM" mode enabled on pin OC0A/PD6
    TCCR0B = 0x03;              // Prescaler = F_CPU / 64

    TC0_OVF_IRQ_ENABLE();       // Enable interrupt on Top count (OVerflow)
}


/*
 * Timer-Counter TC0 (8-bit TMR0 reg.) Interrupt Service Routine
 *
 * The timer will generate an IQR, hence this routine will be executed, when the
 * timer register (TMR0) reaches its MAX count value (255). The timer will reset
 * (TMR0 = 0) and the PWM output will be set HIGH on the next clock pulse.
 *
 * The PWM duty (set by register OCR0A) is updated here.
 * To avoid erratic PWM behaviour, OCR0A should not be written to anywhere else.
 */
ISR( TIMER0_OVF_vect )
{
    unsigned itemp = (255 * PWM_duty_pc) / 100;  // range 0..255
    
    OCR0A = LO_BYTE(itemp);
}

// end of file

