/**
 * File:  appliance_timer_main.c
 *
 * Reference:  AVR Embedded C Tutorial, Lesson 5
 *
 * Example application of software state machine.
 *
 * This version is meant to run on the "AVR-BED" hardware platform.
 * It can be easily adapted to run on the "Atmel 328P X-mini" board -
 * the only differences being CPU clock freq, and peripheral code library.
 *
 * The relay is driven from pin PC5.  The status LED is driven from PD7.
 * The beeper is driven from PD6/OC0A (PWM output).
 * If using an Atmel "X-mini" board, the LCD must be wired according to
 * the schematic diagram in the AVR-Pad X-mini kit documentation pack.
 *
 * The project must have the correct function library loaded...
 * For the X-mini board, substitute lib_avrXmini.a for lib_avrbed.a.
 * The respective library header file must be included in this source file.
 *
 * Version: 1.0    Mike Bauer   June 2020
 */
#include <avr/io.h>

// Comment out whichever does not apply...
#define F_CPU  16000000UL      // X-mini & Nano-BED run at 16 MHz
//#define F_CPU  8000000UL     // AVR-Pad & AVR-BED run at 8 MHz

// These header files contain definitions needed by library functions:
#include <avr/interrupt.h>     // required for interrupt handlers
#include <stdlib.h>
#include <string.h>
// Comment out whichever does not apply...
#include "lib_avrXmini.h"      // def's for X-mini & Nano-BED libraries
//#include "lib_avrbed.h"      // def's for AVR-BED & AVR-Pad libraries

#define BEEP_DURATION_SEC  5      // Set beep duration here

// Define numeric constants for all possible values the state variable
#define SETTING_STATE  0
#define ACTIVE_STATE   1
#define PAUSED_STATE   2
#define BEEPER_STATE   3

// Macros to turn relay driver output on or off --
#define RELAY_OUTPUT_ON()   SET_BIT(PORTC, 5)
#define RELAY_OUTPUT_OFF()  CLEAR_BIT(PORTC, 5)

// Macros to turn status LED on or off and to test its state --
#define STATUS_LED_ON()     SET_BIT(PORTD, 7)
#define STATUS_LED_OFF()    CLEAR_BIT(PORTD, 7)
#define STATUS_LED_IS_ON    TEST_BIT(PORTD, 7)

// Macros to turn beeper (TC0 pulse output) on or off --
#define ENABLE_BEEPER()     TCCR0B = 0x04   // Set TC0 pre-scaler to 1/256
#define DISABLE_BEEPER()    TCCR0B = 0x00   // Inhibit TC0 clock

// Macros to enable & disable Timer-Counter TC0 Compare (A & B) interrupts
#define TC0_OCA_IRQ_ENABLE()   (TIMSK0 |= (1<<OCIE0A))
#define TC0_OCA_IRQ_DISABLE()  (TIMSK0 &= ~(1<<OCIE0A))
#define TC0_OCB_IRQ_ENABLE()   (TIMSK0 |= (1<<OCIE0B))
#define TC0_OCB_IRQ_DISABLE()  (TIMSK0 &= ~(1<<OCIE0B))

// Function prototype declarations
void  SettingState();
void  ActiveState();
void  PausedState();
void  BeeperState();
void  UpdateSettingDisplay();
void  UpdateCountdownDisplay();
void  TC0_Setup(BYTE period, BYTE duty);

short int  state;           // State variable
unsigned  countdown_sec;    // Count-down timer (seconds)
short  hours_set;           // hours setting
short  mins_set;            // minutes setting
BOOL   beeper_enabled;      // TRUE -> beeper active

unsigned long  start1secInterval;    // for 1-second timer
unsigned long  startBeepInterval;    // for beeper timer
unsigned long  startLEDFlashDuty;    // for LED flashing


int main(void)
{
    lcd_initialise();             // Initialize LCD module
    lcd_command(LCD_CLR);         // Send 1-byte command to clear LCD
    lcd_command(LCD_CURSOR_OFF);  // Display ON, Cursor OFF

    // Initialize required I/O pins -------------------------------
    SET_BIT(DDRC, 5);             // PC5 is relay driver output
    SET_BIT(DDRD, 6);             // PD6 is beeper output
    SET_BIT(DDRD, 7);             // PD7 is status LED output

    // Initialise Timer TC1 to use milliseconds() library function
    TC1_initialize();
    GLOBAL_INT_ENABLE();

    // Set beeper freq = 2.5kHz (approx)
#if F_CPU == 8000000   //  8 MHz
    TC0_Setup(12, 3);
#else  // assume F_CPU is 16 MHz
    TC0_Setup(25, 5);
#endif

    // Show start-up menu for setting timer
    lcd_cursor_posn(0, 0);
    lcd_print_string("Setting--  0:00");
    lcd_cursor_posn(1, 0);
    lcd_print_string("A:hours  B:mins");

    state = SETTING_STATE;   // initial state on power-up/reset

    while (TRUE)   // loop forever
    {
        if (isTaskPending_50ms())  ButtonScan(4);   // Every 50ms

        if (state == SETTING_STATE)  SettingState();
        if (state == ACTIVE_STATE)   ActiveState();
        if (state == PAUSED_STATE)   PausedState();
        if (state == BEEPER_STATE)   BeeperState();
    }
}


void  SettingState()
{
    if (button_hit('A'))    // HOURS button hit
    {
        hours_set++;
		if (hours_set >= 6)  { hours_set = 6;  mins_set = 0; }  // cap at 6 hours

        lcd_cursor_posn(1, 0);
        lcd_print_string("C:Clear  D:Start");  // show setting menu
        UpdateSettingDisplay();
    }

    if (button_hit('B'))    // MINS button hit
    {
        if (hours_set == 0)
        {
            if (mins_set < 10)  mins_set++;
            else if (mins_set < 30)  mins_set += 5;
            else  mins_set += 10;
        }
        else  mins_set += 15;

        if (mins_set >= 60)
        {
            mins_set = 0;
            hours_set++;
        }
        if (hours_set >= 6)  { hours_set = 6;  mins_set = 0; }  // cap at 6 hours

        lcd_cursor_posn(1, 0);
        lcd_print_string("C:Clear  D:Start");  // show setting menu
        UpdateSettingDisplay();
    }

    if (button_hit('C'))    // STOP/RESET (CLEAR) button hit
    {
        lcd_cursor_posn(1, 0);
        lcd_print_string("A:hours  B:mins ");  // show start menu
        hours_set = 0;
        mins_set = 0;
        UpdateSettingDisplay();
    }

    if (button_hit('D') && !(hours_set == 0 && mins_set == 0))   // START button hit
    {
        lcd_command(LCD_CLR);   // Prepare display
        lcd_cursor_posn(1, 0);
        lcd_print_string("C:Stop (Pause)");

        countdown_sec = hours_set * 3600 + mins_set * 60;  // seconds to go
        UpdateCountdownDisplay();

        // Activate relay driver output and enter active state
        RELAY_OUTPUT_ON();
        STATUS_LED_ON();
        start1secInterval = milliseconds();  // capture start time
        state = ACTIVE_STATE;
    }
}


void  ActiveState()
{
    if (milliseconds() > (start1secInterval + 1000))  // 1 second elapsed
    {
        countdown_sec--;
        start1secInterval = milliseconds();  // start next 1-sec interval
        UpdateCountdownDisplay();
    }

    if (countdown_sec == 0)  // countdown timer expired
    {
        RELAY_OUTPUT_OFF();
        STATUS_LED_OFF();
        lcd_cursor_posn(0, 0);
        lcd_print_string("Timer expired...");
        lcd_cursor_posn(1, 0);
        lcd_print_string("   Press (C)    ");

        ENABLE_BEEPER();
        startBeepInterval = milliseconds();  // capture start time
        state = BEEPER_STATE;
    }

    if (button_hit('C'))    // STOP/RESET (PAUSE) button hit
    {
        RELAY_OUTPUT_OFF();
        STATUS_LED_OFF();
        lcd_cursor_posn(1, 0);
        lcd_print_string("C:Reset  D:Start");
        startLEDFlashDuty = milliseconds();  // capture start time
        state = PAUSED_STATE;
    }
}


void  PausedState()
{
    if (button_hit('C'))   // STOP/RESET button hit while Paused
    {
        STATUS_LED_OFF();
        lcd_cursor_posn(0, 0);
        lcd_print_string("Setting--       ");  // show start menu
        lcd_cursor_posn(1, 0);
        lcd_print_string("A:hours  B:mins ");
        hours_set = 0;
        mins_set = 0;
        UpdateSettingDisplay();
        state = SETTING_STATE;
    }

    if (button_hit('D'))   // START button hit while Paused
    {
        lcd_cursor_posn(1, 0);
        lcd_print_string("C:Stop (Pause)  ");

        // Re-activate relay driver output and enter active state
        RELAY_OUTPUT_ON();
        STATUS_LED_ON();
        start1secInterval = milliseconds();  // capture start time
        state = ACTIVE_STATE;
    }

    // Flash status LED at 2Hz (half-period = 250ms)
    if (milliseconds() > (startLEDFlashDuty + 250))  // 250ms interval elapsed
    {
        if (STATUS_LED_IS_ON)  STATUS_LED_OFF();  // Toggle LED state
        else  STATUS_LED_ON();
        startLEDFlashDuty = milliseconds();  // for next half-period
    }
}


void  BeeperState()
{
    // Test for Beeper timer expired ---
    if (milliseconds() > (startBeepInterval + BEEP_DURATION_SEC * 1000))
    {
        DISABLE_BEEPER();
    }

    if (button_hit('C'))    // STOP/RESET button hit
    {
        lcd_cursor_posn(0, 0);
        lcd_print_string("Setting--       ");  // show start menu
        lcd_cursor_posn(1, 0);
        lcd_print_string("A:hours  B:mins ");
        hours_set = 0;
        mins_set = 0;
        UpdateSettingDisplay();
        state = SETTING_STATE;
    }
}


void  UpdateSettingDisplay()
{
    lcd_cursor_posn(0, 11);
    lcd_write_char('0' + hours_set);
    lcd_write_char(':');
    lcd_write_char('0' + mins_set / 10);  // "tens" digit
    lcd_write_char('0' + mins_set % 10);  // "ones" digit
}


void  UpdateCountdownDisplay()
{
    BYTE  hours, minutes, seconds;   // local variables for display

    hours = countdown_sec / 3600;   // drop remainder
    minutes = (countdown_sec % 3600) / 60;   // from seconds remaining
    seconds = countdown_sec - (hours * 3600 + minutes * 60);  // remainder

    lcd_cursor_posn(0, 4);
    lcd_write_char('0' + hours);
    lcd_write_char(':');
    lcd_write_char('0' + minutes / 10);
    lcd_write_char('0' + minutes % 10);
    lcd_write_char(':');
    lcd_write_char('0' + seconds / 10);
    lcd_write_char('0' + seconds % 10);
}


//=======================================================================================
// The following functions provide a PWM output signal to drive a passive beeper.

/*
 * Function:  TC0_PWM_Setup()
 *
 * This function initializes Timer TC0 in Normal 'CTC' mode to generate a
 * variable-duty pulse waveform on pin PD6. The output pin is toggled by the
 * Output Compare A and B interrupt service routines.
 *
 * Entry arg. period_reg sets the PWM period (unit = Timer clock cycle).
 * Output pulse duty is set by the global variable: PWM_duty_reg
 * PWM_duty_reg should not be set higher than period_reg.
 *
 * Formula relating pulse output frequency to period_reg value, assuming F_CPU
 * is 16MHz and TC0 pre-scaler is set to 1/256 (TCCR0B = 0x04) -> Tclk = 16us.
 *
 *     freq (Hz) = 1,000,000 / (period_reg x 16)
 *
 * The timer clock is not enabled by this function. To enable the clock,
 * write register TCCR0B = 4  (Pre-scaler control number).
 * To disable the clock again, write TCCR0B = 0.
 */
void  TC0_Setup(BYTE period_reg, BYTE duty_reg)
{
    TCCR0A = 0b00000010;        // Normal CTC mode enabled (TOP = OCR0A)
    TCCR0B = 0;                 // Disable the timer/counter clock
    OCR0A = period_reg - 1;     // Load the period register
    OCR0B = duty_reg;           // Load the duty register
    TC0_OCA_IRQ_ENABLE();       // Enable interrupt on Output Compare A
    TC0_OCB_IRQ_ENABLE();       // Enable interrupt on Output Compare B
}


/*
 * Interrupt Service Routine for Timer-Counter TC0 Output Compare A match.
 * The ISR is executed whenever the timer counter reaches its Top Count, which is
 * the value held in the period register OCR0A. (See TC0 setup function above.)
 *
 * The PWM output pin PD6 is set High.
 */
ISR( TIMER0_COMPA_vect )
{
    SET_BIT(PORTD, 6);
}


/*
 * Interrupt Service Routine for Timer-Counter TC0 Output Compare B match.
 * The ISR is executed whenever the timer counter reaches the PWM duty, which
 * is the value held in the duty register OCR0B.
 *
 * The PWM output pin PD6 is set Low.
 */
ISR( TIMER0_COMPB_vect )
{
    CLEAR_BIT(PORTD, 6);
}


// end of file
