/*
 * File:   serial_numeric_io_main.c
 *
 * Program to test both the functions getString() and putDecimal().
 * The program accepts a string from the user terminal containing two decimal numbers,
 * separated by a comma.
 * The first number is output by putDecimal() with the field width (nplaces) given by
 * the second number in the input string. The output field is delimited by matching
 * square brackets, so that the numeric output format is clear. 
 * The program uses the standard C library function atoi() to convert numbers found in
 * the input string into integer values. 
 *
 * Hardware platform:  Atmel ATmega328P X-mini board
 * Code libraries required:  lib_avrXmini.a and lib_avrSerial.a
 * Author:  M.J.Bauer  2021  [www.mjbauer.biz]
 *
 * Test using PuTTY -- PC terminal app.
 * Configure PuTTY for Serial communications mode. Set baudrate to 38400.
 * Use Windows 'Device Manager' to find the virtual COM port number (COM#)
 * assigned to the 'mEDBG' device. (Look under 'Ports - COM & LPT').
 *
 */
#include <avr/io.h>
#include <stdlib.h>
#include <string.h>
#include "lib_avrXmini.h"
#include "lib_avrSerial.h"  // use pre-built library (.a) or source code (.c)

int  strFindChar(char *s, char ct);


int  main(void)
{
    char  line[20];    // character buffer for input string
    int   commaLocn;   // location of comma in input string
    int   arg1;        // value of 1st "argument" found in input string
    int   arg2;        // value of 2nd "argument" found in input string
    int   width;       // field-width (nplaces) for number output

    UART_init(38400);
    putString("\n\n");  
    putString("Test app for getString() and putDecimal() functions. \n"); 
    putString("At the prompt, enter a decimal number and the field width \n"); 
    putString("for output, separated by a comma.\n"); 
    putString("Example: \n"); 
    putString(">> 99, 4 [Enter] \n"); 
    putString("\n>> ");  // output a prompt
    
    while ( 1 )   // forever...
    {
        if (getString(line, 18))  // got a complete line...
        {
            commaLocn = strFindChar(line, ',');  // search for comma
            if (commaLocn > 0)    // line[] contains a comma and at least 1 digit
            {
                arg1 = atoi(line);   // convert 1st arg (string) to integer value
                arg2 = atoi(&line[commaLocn+1]);   // convert 2nd arg (string) ...
                
                if (arg2 >= 0) width = arg2;  else width = 5;  // default
                
                putChar('[');             // LHS delimiter
                putDecimal(arg1, width);  // Output the number (nplaces = width)
                putChar(']');             // RHS delimiter
                putNewLine();
            }
            else   putString("! Invalid input data -- try again \n"); 

            putString("\n>> ");  // output a prompt
        }
    }
}


/*
 * Function strFindChar() searches for a target character (ct) in a given string (s).
 * If the target char is found, the function returns the location of the character, 
 * i.e. the number of characters preceding the target (which may be zero). 
 *
 * If the target character is not found, the search stops at the end of the input
 * string (NUL term-char), or at location 255, whichever comes first, and the return
 * value is negative (-1) in this case.
 */
int  strFindChar(char *s, char ct)
{
    int   locn = -1;   // assume search fails
    int   i;
    
    for (i = 0;  i < 256;  i++)
    {
        if (s[i] == ct)    // target found...
        {
            locn = i;
            break;         // exit loop
        }   
    }
    
    return  locn;
}
