/**
 * Project:  AVR X'mini audio waveform generator  (Assignment Solution - hide!)
 *
 * This program demonstrates how to use the ATmega328P Timer/Counter module TC0
 * to generate arbitrary waveforms in the audio frequency range (40 ~ 4kHz).
 *
 * AVR X-mini (or Arduino Nano) board wiring information:
 *    1. Pin PD5/OC0B (PWM DAC out) is wired to an external LP filter (fc = 8kHz)
 *    2. Pin PB5 is wired to a LED for status indicator (optional)
 *    3. Pin PC3/ADC3 is wired to the potentiometer arm. (for ADC input)
 *    4. Pins PD2, PD3 & PD4 are wired to push-buttons A, B & C (active Low)
 *
 * Ref: "AVR Waveform Generator notes (H185A)"
 *
 * Ver 2.0   Mike Bauer    July 2020
 *
 * Operating Instructions / Requirements Specification
 * ```````````````````````````````````````````````````````````````````````
 * Press (hold down) button A for > 2 seconds to change UI mode...
 *    Mode 1:  Use pot to set output frequency (40 ~ 4000 Hz)
 *    Mode 2:  Use buttons A (up) and B (down) to set preset frequency
 * Press (tap) Button C to change waveform (sine/triangle/square/sawtooth)
 */
#include <avr/io.h>
#define F_CPU  16000000UL     // X-mini CPU runs at 16 MHz

// These header files contain definitions needed by library functions:
#include <avr/interrupt.h>   // required for interrupt handlers
#include <stdlib.h>
#include <string.h>
#include "lib_avrXmini.h"    // AVR X-mini and Nano-BED library

// Macros to enable & disable Timer-Counter TC0 Overflow (OVF) interrupt
#define TC0_OVF_IRQ_ENABLE()   (TIMSK0 |= (1<<TOIE0))
#define TC0_OVF_IRQ_DISABLE()  (TIMSK0 &= ~(1<<TOIE0))

// Macros to enable & disable Timer-Counter TC0 Output Compare A interrupt
#define TC0_OCA_IRQ_ENABLE()   (TIMSK0 |= (1<<OCIE0A))
#define TC0_OCA_IRQ_DISABLE()  (TIMSK0 &= ~(1<<OCIE0A))

// Define symbolic constants for this app
#define SET_FREQ_USING_BUTTONS   0
#define SET_FREQ_USING_POT       1
#define STEP_UP                 'U'
#define STEP_DOWN               'D'


// ------------  Function prototype declarations  --------------
void  TC0_Setup();
void  ChangeFrequencyPreset(char direction);
void  UpdateFrequencyUsingPot();
void  WaveformUpdate();
void  DisplayUpdate();
void  TC1_Setup();
BOOL  TimerExpired_50ms();

// ------------  Global variables  ----------------------------
unsigned char  *waveTable;         // pointer to active wave-table
unsigned int   oscFreq_Hz;         // Oscillator freq. (range: 40 ~ 4000 Hz)
unsigned int   oscStep;            // Oscillator 'step', 16-bit fixed-point
unsigned int   freqStep;           // Index into freqTable[]
BOOL           UI_mode;            // Frequency setting mode -- 0:Buttons (A & B), 1:Pot.

// Wave-table declarations ...
extern const BYTE  sineWave[];
extern const BYTE  triangleWave[];
extern const BYTE  squareWave[];
extern const BYTE  sawtoothWave[];


int  main(void)
{
	unsigned  buttonApressTime_ms = 0;

    // Configure I/O pins for the application...
    CLEAR_BIT(DDRC, 3);           // Make PC3/ADC3 an input to read pot.
    CLEAR_BIT(DDRD, 2);           // make PD2 an input for push-button
    SET_BIT(PORTD, 2);            // enable internal pull-up on PD2
    SET_BIT(DDRB, 5);             // make PB5 an output to drive status LED
    SET_BIT(DDRD, 5);             // make PD5/OC0B an output for PWM signal

    waveTable = (BYTE *) sineWave;   // default wave-form (sine)
    oscFreq_Hz = 200;                // default osc. freq.
	freqStep = 4;
	UI_mode = SET_FREQ_USING_BUTTONS;  // default UI mode

    // Calculate the oscillator 'step' size using the set output frequency...
    oscStep = ((long) oscFreq_Hz * 160 * 256) / 32000;  // fixed-point value

    TC1_Setup();                  // Set up TC1 for 50ms recurring interval
    TC0_Setup();                  // Set up TC0 for PWM output (Dual-slope mode)
    TC0_OCA_IRQ_ENABLE();         // Enable IRQ for PWM output
    GLOBAL_INT_ENABLE();

    lcd_initialise();             // Initialize LCD module
    lcd_command(LCD_CURSOR_OFF);  // Display ON, Cursor OFF
    lcd_cursor_posn(0, 0);
    lcd_print_string("Wave: ");
    lcd_cursor_posn(1, 0);
    lcd_print_string("Freq: ");

    while (TRUE)  // loop forever
    {
        if (TimerExpired_50ms())
        {
            // The following periodic tasks are executed every 50 ms:
            ButtonScan(3);
            DisplayUpdate();

			if (UI_mode == SET_FREQ_USING_POT)  UpdateFrequencyUsingPot();

			if (button_pressed('A'))   // button A held down
			{
				buttonApressTime_ms += 50;
				if (buttonApressTime_ms == 2000)  UI_mode = !UI_mode;   // toggle mode
			}
			else  buttonApressTime_ms = 0;
        }

		if (button_hit('C'))   // Change wave-form
		{
			if (waveTable == sineWave)  waveTable = (BYTE *) triangleWave;
			else if (waveTable == triangleWave)  waveTable = (BYTE *) squareWave;
			else if (waveTable == squareWave)  waveTable = (BYTE *) sawtoothWave;
			else  waveTable = (BYTE *) sineWave;
		}

		if (UI_mode == SET_FREQ_USING_BUTTONS)
		{
			if (button_hit('A'))  ChangeFrequencyPreset(STEP_UP);
			if (button_hit('B'))  ChangeFrequencyPreset(STEP_DOWN);
		}
    }
}


/*
 * Function:  TC0_Setup()
 *
 * This function initializes Timer-Counter TC0 in 8-bit PWM mode (Phase Correct)
 * to generate a variable-duty pulse waveform on pin OC0B (= PD5).
 *
 * The timer "Output Compare A" function is used to set the PWM pulse period
 * equal to the sample interval (31.25us) for a sample rate of 32 kHz.
 *
 * The timer "Output Compare B" function is used to set the PWM pulse duty.
 *
 * The timer clock pre-scaler is disabled (N = 1).
 * Assuming the CPU clock freq. is 16.000 MHz, register OCR0A is set to 249 for
 * a timer period of 500 clocks, so the PWM output freq. will be 16MHz / 500 = 32kHz.
 *
 */
void  TC0_Setup()
{
    TCCR0A = 0b00100001;        // Phase Correct PWM mode...
                                // OC0A/PD6 pin not used;  OC0B/PD5 is PWM output
    TCCR0B = 0b00001001;        // Wave-form generator mode = 5;  Prescaler N = 1

    OCR0A = 249;                // load OCR0A register for PWM half-period = 250 clocks
    OCR0B = 125;                // Load OCR0B register for PWM duty = 125 clocks (50%)
}


/*
 * Timer-Counter TC0 Interrupt Service Routine
 *
 * The timer will generate an IRQ, hence this routine will be executed, when the
 * timer count register (TMR0) reaches the 'TOP' count value (= OCR0A).
 * The counter will reverse direction, i.e. change from Up to Down or vice-versa.
 *
 * The PWM duty (set by register OCR0B) is updated here with the next sample.
 *
 * The wave-table oscillator uses 16-bit fixed-point arithmetic. The high-order
 * byte represents the integer part; the low-order byte is the fractional part.
 * A detailed description of the wave-table oscillator algorithm is given in a
 * separate document (see Reference).
 */
ISR( TIMER0_COMPA_vect )
{
    static unsigned int  oscAngle = 0;  // fixed-point (8:8 bits)

    unsigned char sample;     // range 1 .. 249  (= 125 +/- 124)
    unsigned int  index;      // wave-table index

    // Wave-table oscillator implementation
    index = HI_BYTE(oscAngle);   // index = integer part of oscAngle
    sample = waveTable[index];   // index = 0..159
    oscAngle = (oscAngle + oscStep) % ((unsigned)160 * 256);  // next sample point

    OCR0B = sample;   // update PWM duty with sample value
}


void  ChangeFrequencyPreset(char direction)
{
	// Table of 16 frequency preset values:
	static unsigned freqTable[] =
		{ 40, 50, 80, 100, 160, 200, 250, 400, 500, 800, 1000, 1600, 2000, 2500, 4000, 5000 };

	if (direction == STEP_UP && freqStep < 15)  freqStep++;
	if (direction == STEP_DOWN && freqStep > 0)  freqStep--;

	oscFreq_Hz = freqTable[freqStep];

	// Set the oscillator "phase angle step" size according to oscFreq_Hz
	oscStep = ((long) oscFreq_Hz * 160 * 256) / 32000;  // fixed-point value
}


/*
 * Function:  UpdateFrequencyUsingPot()
 *
 * This function reads the pot setting via ADC input ADC3/PC3.
 * The ADC reading is used to set the oscillator freq.
 * Called at least 5 times per second.

 noiseFilterAccum -= noiseFilterOut;   // last sample
 noiseFilterAccum += noiseSample;
 noiseFilterOut = noiseFilterAccum / m_NoiseFilter_K;

 *
 * Bonus points for smoothing the pot reading (e.g. by averaging past values)!
 */
void  UpdateFrequencyUsingPot()
{
	static  long  filterAccum;   // filter accumulator (sum)
	static  long  lastReading;   // filter output (average)

    int  potReading = ADC_ReadInput(3) / 4;   // Read ADC3 input

	// Apply rolling average filter (1st order IIR) to smooth pot reading...
	filterAccum = filterAccum - lastReading;
	filterAccum = filterAccum + potReading;
	lastReading = filterAccum / 4;

    // Apply square-law transform to obtain osc. frequency setting.
    // Note: lastReading is 32 bits to avoid arithmetic overflow!
    oscFreq_Hz = (4000 * lastReading * lastReading) / ((long) 250 * 250);

	if (oscFreq_Hz < 40)  oscFreq_Hz = 40;
	if (oscFreq_Hz > 4000)  oscFreq_Hz = 4000;

    // Set the oscillator "phase angle step" size according to oscFreq_Hz
    oscStep = ((long) oscFreq_Hz * 160 * 256) / 32000;  // fixed-point value
}


/*
 * Function:  DisplayUpdate()
 *
 * This function updates information displayed on the LCD, if anything has
 * changed since the last update.  Called at least 5 times per second.
 */
void  DisplayUpdate()
{
    static BYTE  *previousWaveTable = NULL;    // NULL forces update on startup
    static unsigned int previousOscFreq = 0;   // Zero forces update on startup
    char   strBuff[16];
    char   numBlanks = 0;

    if (waveTable != previousWaveTable)  // Waveform has changed
    {
        lcd_cursor_posn(0, 6);
        if (waveTable == sineWave)  lcd_print_string("Sine    ");
        else if (waveTable == triangleWave)  lcd_print_string("Triangle");
        else if (waveTable == squareWave)  lcd_print_string("Square  ");
        else if (waveTable == sawtoothWave)  lcd_print_string("Sawtooth");
        previousWaveTable = waveTable;
    }

    if (oscFreq_Hz != previousOscFreq)  // Osc. Freq. has changed
    {
        itoa(oscFreq_Hz, strBuff, 10);
        numBlanks = 4 - strlen(strBuff);   // spaces for padding
        lcd_cursor_posn(1, 6);             // erase 4 chars from column 6
        lcd_print_string("    ");
        lcd_cursor_posn(1, 6 + numBlanks);
        lcd_print_string(strBuff);
        lcd_cursor_posn(1, 11);
        lcd_print_string("Hz");
        previousOscFreq = oscFreq_Hz;
    }

	lcd_cursor_posn(1, 15);
	if (UI_mode == SET_FREQ_USING_POT)  lcd_write_char('o');
	else  lcd_write_char(' ');
}


/*
 * Set up timer TC1 in Normal mode -- count up, rollover, no interrupt.
 * Pre-scaler:  F_CPU / 64  -->  Tclk = 4us (@ F_CPU = 16MHz)
 * Count == 12500 -> 50,000us -> 50ms
 */
void  TC1_Setup()
{
	TCCR1B = 0;            // Disable TC1 during setup
    TCCR1A = 0;            // Normal mode
	TCNT1 = 0;             // Reset timer count reg.
    TCCR1B = 0b00000011;   // Enable timer;  Pre-scaler = 1/64;
}


/*
 * Timer TC1 "service routine" -- Polling mode (non-interrupt).
 * This function must be called frequently (< 5ms intervals) from the main loop.
 *
 * The timer count register is checked;  if it is > 50ms, i.e. timer expired,
 * then the timer is reset (count = 0) and the function returns TRUE.
 * Otherwise, it returns FALSE.
 */
BOOL  TimerExpired_50ms()
{
	BOOL  result = FALSE;
	unsigned  count = TCNT1;

    if (count >= 12500)
	{
		result = TRUE;
		TCNT1 = 0;
	}

	return  result;
}


const BYTE  sineWave[] =
{
    128,  131,  135,  139,  143,  147,  151,  155,  158,  162,
    166,  169,  173,  176,  180,  183,  186,  189,  192,  195,
    198,  201,  204,  206,  208,  211,  213,  215,  217,  218,
    220,  221,  223,  224,  225,  226,  226,  227,  227,  227,
    228,  227,  227,  227,  226,  226,  225,  224,  223,  221,
    220,  218,  217,  215,  213,  211,  208,  206,  204,  201,
    198,  195,  192,  189,  186,  183,  180,  176,  173,  169,
    166,  162,  158,  155,  151,  147,  143,  139,  135,  131,
    128,  125,  121,  117,  113,  109,  105,  101,   98,   94,
    90,   87,   83,   80,   76,   73,   70,   67,   64,   61,
    58,   55,   52,   50,   48,   45,   43,   41,   39,   38,
    36,   35,   33,   32,   31,   30,   30,   29,   29,   29,
    28,   29,   29,   29,   30,   30,   31,   32,   33,   35,
    36,   38,   39,   41,   43,   45,   48,   50,   52,   55,
    58,   61,   64,   67,   70,   73,   76,   80,   83,   87,
    90,   94,   98,  101,  105,  109,  113,  117,  121,  125
};


const  BYTE  triangleWave[] =
{
    128,  130,  133,  135,  138,  140,  143,  145,  148,  150,
    153,  155,  158,  160,  163,  165,  168,  170,  173,  175,
    178,  180,  183,  185,  188,  190,  193,  195,  198,  200,
    203,  205,  208,  210,  213,  215,  218,  220,  223,  225,
    228,  226,  223,  221,  218,  216,  213,  211,  208,  206,
    203,  201,  198,  196,  193,  191,  188,  186,  183,  181,
    178,  176,  173,  171,  168,  166,  163,  161,  158,  156,
    153,  151,  148,  146,  143,  141,  138,  136,  133,  131,
    128,  126,  123,  121,  118,  116,  113,  111,  108,  106,
    103,  101,   98,   96,   93,   91,   88,   86,   83,   81,
    78,   76,   73,   71,   68,   66,   63,   61,   58,   56,
    53,   51,   48,   46,   43,   41,   38,   36,   33,   31,
    28,   30,   33,   35,   38,   40,   43,   45,   48,   50,
    53,   55,   58,   60,   63,   65,   68,   70,   73,   75,
    78,   80,   83,   85,   88,   90,   93,   95,   98,  100,
    103,  105,  108,  110,  113,  115,  118,  120,  123,  125
};


const BYTE  squareWave[] =
{
    228,  228,  228,  228,  228,  228,  228,  228,  228,  228,
    228,  228,  228,  228,  228,  228,  228,  228,  228,  228,
    228,  228,  228,  228,  228,  228,  228,  228,  228,  228,
    228,  228,  228,  228,  228,  228,  228,  228,  228,  228,
    228,  228,  228,  228,  228,  228,  228,  228,  228,  228,
    228,  228,  228,  228,  228,  228,  228,  228,  228,  228,
    228,  228,  228,  228,  228,  228,  228,  228,  228,  228,
    228,  228,  228,  228,  228,  228,  228,  228,  228,  228,
    28,   28,   28,   28,   28,   28,   28,   28,   28,   28,
    28,   28,   28,   28,   28,   28,   28,   28,   28,   28,
    28,   28,   28,   28,   28,   28,   28,   28,   28,   28,
    28,   28,   28,   28,   28,   28,   28,   28,   28,   28,
    28,   28,   28,   28,   28,   28,   28,   28,   28,   28,
    28,   28,   28,   28,   28,   28,   28,   28,   28,   28,
    28,   28,   28,   28,   28,   28,   28,   28,   28,   28,
    28,   28,   28,   28,   28,   28,   28,   28,   28,   28
};


const BYTE  sawtoothWave[] =
{
    28,   29,   30,   31,   33,   34,   35,   36,   38,   39,
    40,   41,   43,   44,   45,   46,   48,   49,   50,   51,
    53,   54,   55,   56,   58,   59,   60,   61,   63,   64,
    65,   66,   68,   69,   70,   71,   73,   74,   75,   76,
    78,   79,   80,   81,   83,   84,   85,   86,   88,   89,
    90,   91,   93,   94,   95,   96,   98,   99,  100,  101,
    103,  104,  105,  106,  108,  109,  110,  111,  113,  114,
    115,  116,  118,  119,  120,  121,  123,  124,  125,  126,
    128,  129,  130,  131,  133,  134,  135,  136,  138,  139,
    140,  141,  143,  144,  145,  146,  148,  149,  150,  151,
    153,  154,  155,  156,  158,  159,  160,  161,  163,  164,
    165,  166,  168,  169,  170,  171,  173,  174,  175,  176,
    178,  179,  180,  181,  183,  184,  185,  186,  188,  189,
    190,  191,  193,  194,  195,  196,  198,  199,  200,  201,
    203,  204,  205,  206,  208,  209,  210,  211,  213,  214,
    215,  216,  218,  219,  220,  221,  223,  224,  225,  226
};


