/**
* File:      CLI_support.c
* Created:   2020-09-24  Mike Bauer
* Version:   1.0
*
* This file provides functions to implement a simple Command-Line User Interface (CLI),
* for use with the serial port (UART) on various AVR micro-controller platforms.
*
* Functions here can't be used to create a re-usable library, because much of the
* code is application-specific, i.e. it needs to be modified to suit each application.
*/
#include "CLI_support.h"

//    CLI data
//`````````````````````````````````````````````````````````````````````````````
char  cmdLine[CMD_LINE_MAX_LEN + 2];   // Command Line buffer (array)

// Strings extracted from the last command line entered...
char  cmdName[CLI_ARG_MAX_LEN + 1];    // Command name (string)
char  argStr1[CLI_ARG_MAX_LEN + 1];    // Command argument string #1 (if any)
char  argStr2[CLI_ARG_MAX_LEN + 1];    // Command argument string #2 (if any)
char  argStr3[CLI_ARG_MAX_LEN + 1];    // Command argument string #3 (if any)

char  *cmdError = "! Command error.\n";    // Frequently used message!

// Nick-names of parameters corresponding to values in array paramValue[]
// A NULL string is appended to mark the end of the list.
char  *paramName[] = { "Pgain", "Igain", "calT1", "calT2", NULL };

//_____________________________________________________________________________

/*
 * Function:  ExtractArg(char *mess, BYTE offset, char *dest)
 *
 * Extracts (copies) a sub-string from a character buffer (mess) into a
 * destination array (dest[]), beginning at a given position in mess[].
 * A "sub-string" is normally terminated by a SPACE or NUL code.
 *
 * The return value is the offset (index) of the next sub-string following the
 * sub-string extracted, if any, or else the offset of the NUL terminator.
 *
 * Refer to header file "CLI_support.h" for a more detailed description.
 */
BYTE  ExtractArg(char *mess, BYTE offset, char *dest)
{
	int   index = offset;        // index into input array, mess[]
	int   outdex = 0;            // index into output array, dest[]
	BYTE  count = 0;

	if (mess[index] < SPACE)  return index;  // end-of message -- bail

	// advance to first non-space char, if any...
	while (mess[index] == ' ')  { index++; }

	while (count < CLI_ARG_MAX_LEN)   // copy up to CLI_ARG_MAX_LEN chars to dest[]
	{
		if (mess[index] <= SPACE) break;  // control code or space -- bail

		dest[outdex++] = mess[index++];   // copy 1 char
		dest[outdex] = NUL;               // add string terminator
		count++;
	}

	if (mess[index] < SPACE)  return index;  // end-of message -- bail

	// advance to next non-space char
	while (mess[index] == SPACE)  { index++; }

	return  (BYTE) index;
}


/*
 * Function:   strMatch()
 *
 * Purpose:    Compares two NUL-terminated strings, each up to 255 chars in length.
 *             Returns TRUE if the strings have the same contents, up to the terminator
 *             of either string, except that the comparison is not case-sensitive.
 */
BOOL  strMatch(char *str1, char *str2)
{
    char   c1, c2;
	BYTE   i;
    BOOL   result = TRUE;  // assume until found otherwise

    for (i = 0;  i < 255;  i++)
    {
	    c1 = tolower( str1[i] );
	    c2 = tolower( str2[i] );
	    if (c1 != c2)  result = FALSE;
	    if (c1 == 0 || c2 == 0)  break;  // found NUL -- exit
    }

    return result;
}


/*
 * Function:  CommandLineParser(char  *cmdLine)
 *
 * Purpose:  Called whenever a complete command line (string) is received, this
 * function analyzes the string and extracts the command name and argument strings.
 * The respective command function (if any exists) is then executed.
 *
 * Refer to header file "CLI_support.h" for a more detailed description.
 */
void  CommandLineParser(char  *cmdLine)
{
	BYTE  offset = 0;   // marker of next argument in cmdLine[]
	BYTE  argCount;     // Number of cmd "arguments" incl. cmdName
	BYTE  cmdLineLength = strlen(cmdLine);

	// Clear argument strings pending extraction from cmdLine:
	argStr1[0] = argStr2[0] = argStr3[0] = NUL;

	if (cmdLineLength == 0)   // command line is empty...
	{
		putString("\n> ");  // prompt for next command & bail
		return;
	}

	argCount = 1;  // The command name itself is counted as an "argument"

	// Extract command name from the line buffer
	offset = ExtractArg(cmdLine, 0, cmdName);

	// If there's an argument, extract it from the line buffer
	if (offset < cmdLineLength)
	{
		offset = ExtractArg(cmdLine, offset, argStr1);  // get 1st arg
		argCount++;
	}
	// If there's a 2nd argument, extract it from the line buffer
	if (offset < cmdLineLength)
	{
		offset = ExtractArg(cmdLine, offset, argStr2);  // get 2nd arg
		argCount++;
	}
	// If there's a 3rd argument, extract it from the line buffer
	if (offset < cmdLineLength)
	{
		offset = ExtractArg(cmdLine, offset, argStr3);  // get 3rd arg
		argCount++;
	}
/***
	// Temp. for debug...
	putString("argCount = ");  putDecimal(argCount, 1);  putNewLine();
	putString("cmdName  = ");  putString(cmdName);  putNewLine();
	putString("argStr1  = ");  putString(argStr1);  putNewLine();
	putString("argStr2  = ");  putString(argStr2);  putNewLine();
	putString("argStr3  = ");  putString(argStr3);  putNewLine();
***/
	// Try to match the received command with a defined command name.
	// If found, call the respective command function.
	if (strMatch(cmdName, "help"))  HelpCommand(argCount);
	else if (strMatch(cmdName, "default"))  DefaultCommand(argCount);
	else if (strMatch(cmdName, "set"))  SetCommand(argCount);
	else if (strMatch(cmdName, "calib"))  CalibrateCommand(argCount);
	//
	// Add more command options here as required...
	//
	else  { putString("! Undefined command: ");  putString(cmdName);  putNewLine(); }

	putString("\n> ");  // prompt for next command
}


//             Application-specific command functions
//```````````````````````````````````````````````````````````````````````````````````
/*
 * Command function: DoHelpCommand()
 *
 * Regardless of argument count, this function outputs help on CLI usage
 * and it lists the set of available commands.
 */
void  HelpCommand(BYTE argCount)
{
	putString("Available commands:\n");
	putString("    help  default  set  calib  \n");
	putString("To see cmd usage info, type cmd name with '?'\n");
	putString("Example:  set ? [Enter] \n");
}


/*
 * Command function: DoDefaultCommand()
 *
 * The "default" command restores user-settable parameters to default values.
 * If called from the CommandLineParser() function, the "default" command will be
 * executed only if the command line has a dollar sign ($) as its argument.
 * This requirement is intended to avoid accidental defaulting by the user.
 */
void  DefaultCommand(BYTE argCount)
{
	if (argCount == 0 || (argCount >= 2 && argStr1[0] == '$'))
	{
		// todo:  assign sensible default values !
		paramValue[0] = 1100;   // Pgain x 1000  ?
		paramValue[1] = 1234;   // Igain x 1000  ?
		paramValue[2] = 250;    // calT1 x 10 deg.C
		paramValue[3] = 500;    // calT2 x 10 deg.C

		StorePersistentParams();   // Save to EEPROM
		putString("* Param's defaulted.\n");
		SetCommand(1);   // List param's
	}
	else  putString("! Usage:  default $ \n");
}


/*
 * Command function: DoSetCommand()
 *
 * The "set" command, if entered without any argument, outputs a list of all
 * user-settable parameters -- nicknames and values.
 *
 * To update a parameter value, the command line must have two more arguments:
 * the nickname of the parameter to be changed and its new value.
 * If a param is successfully updated, all param's & values will be listed.
 *
 * The function does not do a "sanity check" on the param value entered,
 * however it could be easily extended to do so.
 */
void  SetCommand(BYTE argCount)
{
	int  i = 0;

	if (argCount == 2 && argStr1[0] == '?')  // help wanted
	{
		putString("Set or view a param value.\n");
		putString("Usage:  set [param] [value]\n");
		putString("where <param> is one of:  Pgain, Igain, calT1, calT2\n");
		return;
	}

	if (argCount == 2 || argCount >= 4)  // Wrong number of arg's
	{
		putString(cmdError);
		return;    // bail
	}

	// Search for param name ... (Search will fail if argCount == 1)
	for (i = 0;  i < NUMBER_OF_PARAMS;  i++)
	{
		if (strMatch(argStr1, paramName[i]))  // found match...
		{
			paramValue[i] = atoi(argStr2);    // convert arg string to integer
			StorePersistentParams();   // Save updated param to EEPROM
			break;  // done
		}
	}

	if (argCount == 1 || i < NUMBER_OF_PARAMS)  // List param's & values
	{
		for (i = 0;  i < NUMBER_OF_PARAMS;  i++)
		{
			putChar(ASCII_TAB);
			putString(paramName[i]);
			putChar(ASCII_TAB);
			putDecimal(paramValue[i], 5);
			putNewLine();
		}
	}
	else  putString("! No such param.\n");
}


/*
 * Command function: DoCalibrateCommand()
 *
 * The "calib" command uses the current values of the calibration param's, calT1 and
 * calT2, to re-calculate the temperature sensor error compensation characteristics.
 */
void  CalibrateCommand(BYTE argCount)
{
	putString(cmdError);  // todo !
}



// end-of-file
