/**
* File:  CLI_support.h
*
* Definitions & declarations for the CLI support functions in "CLI_support.c"
*
* Created:   2020-09-24  Mike Bauer
*
* This header file must be #included in all application source code files.
*/
#ifndef  _CLI_SUPPORT_H    // Avoid multiple inclusion of this file
#define  _CLI_SUPPORT_H

#include <avr/io.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "lib_avrXmini.h"      // def's for X-mini library functions
#include "lib_avrSerial.h"     // def's for AVR serial library functions

#define NUMBER_OF_PARAMS   4   // Number of user-settable param's defined
#define CMD_LINE_MAX_LEN  64   // Maximum length of CLI command line (chars)
#define CLI_ARG_MAX_LEN    8   // Maximum length of CLI cmd arg's (chars)


//  Functions which are provided by the application program
// ```````````````````````````````````````````````````````````
extern BOOL  FetchPersistentParams();
extern void  StorePersistentParams();


//  CLI  global variables (in case needed by another code module)
//````````````````````````````````````````````````````````````````````````````
extern char  cmdLine[];     // Command Line buffer (array)
extern char  cmdName[];     // Command name (string)
extern char  argStr1[];     // Command argument string #1 (if any)
extern char  argStr2[];     // Command argument string #2 (if any)
extern char  argStr3[];     // Command argument string #3 (if any)

//  Application-specific globals
//`````````````````````````````````````````````````````````````````````````
// User-settable parameters... Working copies of param's stored in EEPROM
extern int   paramValue[];

// Nick-names of parameters corresponding to values in array paramValue[]
extern char  *paramName[];


/*  Function prototype declarations of functions in file: "CLI_support.c"
 * ```````````````````````````````````````````````````````````````````````````
 *
 * Function:  ExtractArg(char *mess, BYTE offset, char *dest)
 *
 * Extracts (copies) a sub-string from a character buffer (mess) into a
 * destination array (dest[]), beginning at a given position in mess[].
 * A "sub-string" is normally terminated by a SPACE or NUL code.
 *
 * The output string (dest) cannot exceed 10 printable characters in size.
 * If a sub-string in the source buffer (mess) is longer than 10 characters,
 * only the first 10 will be copied into the output string.
 * The output string is always NUL terminated.
 *
 * The starting position (index) is specified by the argument 'offset'.
 * Copying stops and the function returns when a SPACE (0x20) or a control code
 * e.g. NUL terminator (0) is found following the sub-string extracted.
 * In any case, there will be no spaces copied to the output buffer, dest[].
 *
 * If there's a SPACE at the starting position, the offset will be incremented
 * until a non-space printable character, or a NUL terminator, is found.
 *
 * The return value is the offset (index) of the next sub-string following the
 * sub-string extracted, if any, or else the offset of the NUL terminator.
 */
BYTE  ExtractArg(char *mess, BYTE offset, char *dest);


/*
 * Function:   strMatch()
 *
 * Purpose:    Compares two NUL-terminated strings, each up to 255 chars in length.
 *             Returns TRUE if the strings have the same contents, up to the terminator
 *             of either string, except that the comparison is not case-sensitive.
 *
 * Entry args: str1, str2 :  identifiers of the two strings to be compared
 *
 * Return val: TRUE if string #1 matches string #2 (case-insensitive), else FALSE.
 */
BOOL  strMatch(char *str1, char *str2);


/*
 * Function:  CommandLineParser(char  *cmdline)
 *
 * Purpose:  Called whenever a complete command line (string) is received, this
 * function analyzes the string and extracts the command name and argument strings,
 * if any from it and copies these to separate character arrays. The number of
 * command line "arguments" (including the command name) is counted.
 *
 * The function compares the input command name with a list of defined commands.
 * If a match is found, the respective command function is called.
 */
void  CommandLineParser(char  *cmdline);


//  Application-specific command functions (prototypes)
//````````````````````````````````````````````````````````````````````````````
void  HelpCommand(BYTE argCount);
void  DefaultCommand(BYTE argCount);
void  SetCommand(BYTE argCount);
void  CalibrateCommand(BYTE argCount);


#endif  // _CLI_SUPPORT_H
