/**
* Solution:  C_Lesson_6_soln
* Project:   CLI-support-test
* File:      main.c
* Created:   2020-09-24  Mike Bauer
* Version:   1.0
*
* This program tests and demonstrates functions provided in the AVR serial comm's library,
* "lib_avrSerial.a".  A simple Command-Line User Interface (CLI) is implemented.
* Functions to support the CLI are provided in a separate source file: "CLI_support.c".
*
* The code may be used as a template for applications incorporating a CLI accessible via
* the AVR asynchronous serial comm port (UART).
*/
#include <avr/io.h>
#define F_CPU  16000000UL     // CPU system clock frequency (Hz)

// These header files contain definitions needed by library functions:
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "lib_avrXmini.h"      // def's for X-mini library functions
#include "lib_avrSerial.h"     // def's for AVR serial library functions
#include "CLI_support.h"       // CLI function module


// Functions defined in this source file...
// ```````````````````````````````````````````
BOOL  FetchPersistentParams();  // Copy user-settable param's from EEPROM
void  StorePersistentParams();  // Store user-settable param's in EEPROM

//    Application-specific globals
//`````````````````````````````````````````````````````````````````````````
// User-settable parameters... Working copies of param's stored in EEPROM
int   paramValue[NUMBER_OF_PARAMS + 1];
//
//_________________________________________________________________________


int  main(void)
{
	static unsigned long  LEDturnOffTime;

	SET_BIT(DDRB, 5);   // PB5 is output for heartbeat LED

	lcd_initialise();
	lcd_command(LCD_CURSOR_OFF);
	lcd_cursor_posn(0, 0);
	lcd_print_string("CLI test    v1.0");
	lcd_cursor_posn(1, 0);
	lcd_print_string("Use COM terminal");

	TC1_initialise();
	GLOBAL_INT_ENABLE();

	UART_init(57600);   // Initialize UART, set baud rate
	putString("\n\n");
	putString("AVR X'mini CLI, v1.0 \n");  // send startup message

	// Fetch settable param's from EEPROM and check for valid data
	if (FetchPersistentParams() == FALSE)
	{
		putString("! EEPROM data error...\n");
		DefaultCommand(0);
	}

	HelpCommand(0);        // print help about command line usage
	putString("\n> ");     // prompt for first command

	while (TRUE)  // loop forever
	{
		if (isTaskPending_500ms())  // True every 0.5 second
		{
			SET_BIT(PORTB, 5);    // Turn heartbeat LED ON
			LEDturnOffTime = milliseconds() + 100;
		}

		// Check if the LED on-time (duty) has reached 100ms...
		if (milliseconds() >= LEDturnOffTime)  CLEAR_BIT(PORTB, 5);  // Turn LED off

		if (isTaskPending_50ms())
		{
			ButtonScan(4);    // ButtonScan called every 50ms
			;                 // Add any other 50ms tasks here
			;
		}

		if (getString(cmdLine, CMD_LINE_MAX_LEN))   // Have complete command line...
		    CommandLineParser(cmdLine);             // Execute the command function
	}
}


// Copy user-settable param's from EEPROM
//
BOOL  FetchPersistentParams()
{
	BOOL  result = TRUE;  // assume data valid

	EEPROM_ReadArray((BYTE *) paramValue, sizeof(paramValue));

	if (paramValue[0] == 0xFFFF)  result = FALSE;  // EEPROM erased!

	return result;
}


// Store user-settable param's in EEPROM
//
void  StorePersistentParams()
{
	EEPROM_WriteArray((BYTE *) paramValue, sizeof(paramValue));
}

// end-of-file
