/**
 * File:  lib_avrSerial.c
 *
 * <!>  Assignment Solution -- Hide from students!!
 * ````````````````````````````````````````````````````
 * Serial Port Function Library for the Atmel ATmega328P "X-mini" dev board.
 * Note: This library provides higher-level functions than "lib_avrXmini.a".
 *
 * Refer to corresponding header file "lib_avrSerial.h" for detailed function
 * descriptions and usage information.
 *
 * Author:   Mike bauer   2020:09:25
 # Version:  1.0
 */
#include "lib_avrSerial.h"     // def's for AVR serial library functions

/**
 * Function:  putDecimal()
 *
 * Outputs via serial port an unsigned integer (uval) as a decimal number, up to 5 digits,
 * in a field of a given size (nplaces) at least, padded if necessary with leading spaces.
 */
void  putDecimal(unsigned uval, BYTE nplaces)
{
    char   digit[6];   // ASCII result string, digit[0] is LSD
    signed char  idx;
    char   c;

    if (nplaces > 6)  nplaces = 6;
    if (nplaces < 1)  nplaces = 1;

	// This loop does the actual conversion, replacing _leading_ zeros with spaces,
	// except the LSD. Conversion begins with the least significant digit (LSD).
    for (idx = 0;  idx < 6;  idx++)
    {
	    c = '0' + (uval % 10);
		if (uval == 0 && c == '0' && idx != 0)  c = ' ';  // leading zero
	    digit[idx] = c;
	    uval = uval / 10;
    }

	// This loop outputs digits (or spaces) in reverse order, i.e. MSD to LSD.
	// Each char is output only if it's a non-zero digit, or a pad space within the
	// allowed field width (idx < nplaces).
    for (idx = 5;  idx >= 0;  idx--)
    {
	    c = digit[idx];  // may be a leading space
		if (isdigit(c) || idx < nplaces)  putChar(c);
    }
}


/**
 * Function:  getString()
 *
 * Accepts a string (str) via serial port input, up to a limited length (maxlen).
 * This is a non-blocking function which may take multiple calls to receive the string;
 * i.e. it does not wait for user input.
 *
 * When an ASCII CR (13) code is received, the function returns TRUE;  else FALSE.
 * Primitive line editing capability is implemented by processing ASCII backspace (BS)
 * and cancel line (CAN) codes. (See header file for more detail.)
 */
BOOL  getString(char *str, BYTE maxlen)
{
	static  int   index;       // index into array mess[] - remembered
	static  BYTE  count;       // number of chars in output string - remembered
	BYTE  rxb;                 // received byte (character)
	BOOL  status = FALSE;      // return value

	if (UART_RxDataAvail())    // a byte is available in the UART RX register
	{
		rxb = UART_GetByte();  // capture it

		if (rxb == ASCII_CR)   // CR code -- got complete command line
		{
			putNewLine();      // Echo CR + LF
			str[index] = 0;    // add NUL terminator
			index = 0;         // reset index for next line input
			count = 0;
			status = TRUE;
		}
		else if (rxb >= SPACE && count < maxlen) // printable char
		{
			putChar(rxb);      // echo rxb back to user
			str[index] = rxb;
			index++;
			count++;
		}
		else if (rxb == ASCII_BS && count != 0)  // Backspace
		{
			putChar(ASCII_BS);   // erase offending char
			putChar(SPACE);
			putChar(ASCII_BS);   // re-position cursor
			index--;             // remove last char in buffer
			count--;
		}
		else if (rxb == ASCII_CAN || rxb == ASCII_ESC)  // Cancel line
		{
			putString(" ^X^ \n");
			putString("> ");        // prompt
			index = 0;
			count = 0;
		}
	}

	return  status;
}

