/**
* Project:  AVR X-mini clock  (v2)
* File:     tod_clock_soln_main.c
*
* The LCD module and push-buttons must be wired according to the library requirements.
* (Refer to AVR board code library documentation.)
*
* Operating instructions:
* To set the time, hold down button 'A' and press 'B' to set hours, 'C' to set minutes.
* Release button 'A' to show time-of-day again (with seconds zeroed).
* To switch between 24-hour and 12-hour (am/pm) display mode, press button 'D'.
*
* Author:    Mike Bauer   2020-09-17
*
*/
#include <avr/io.h>
#define F_CPU  16000000UL     // CPU system clock frequency (Hz)

// These header files contain definitions needed by library functions:
#include <stdlib.h>
#include <string.h>
#include "lib_avrXmini.h"     // def for code library functions

// Function prototype declarations...
// `````````````````````````````````````
void  DisplayTimeOfDay();
void  SettingTimeOfDay();


// ------------  Global variables  --------------------------------------
char  rtcHour, rtcMinutes, rtcSeconds;   // time-of-day clock "registers" (24-hour mode)
unsigned long  LEDturnOffTime;    // Time to turn LED off (ms since Reset)
char  display_mode;               // 0 -> 24-hour mode;  !0 -> 12-hour mode


int  main(void)
{
    BYTE toggle = 0;

    rtcHour = 10;      // Initialise time-of-day "registers"
    rtcMinutes = 55;
    rtcSeconds = 00;
	display_mode = 0;  // default mode is 24-hour

    lcd_initialise();             // Initialize LCD module
    lcd_command(LCD_CLR);         // Send 1-byte command to clear LCD
    lcd_command(LCD_CURSOR_OFF);  // Display ON, Cursor OFF

    // Configure I/O pins (data directions) for push-buttons and LED(s)
	SET_BIT(DDRB, 5);

    TC1_initialize();             // Initialize milliseconds() timer function
    GLOBAL_INT_ENABLE();

    lcd_cursor_posn(0, 0);
    lcd_print_string("AVR X'mini clock");   // string to go on top line

    while (TRUE)  // loop forever
    {
        if (isTaskPending_500ms())  // True every 0.5 second
        {
            toggle++;

            if (toggle % 2)   // <---- this happens every 1 second (500ms x 2)
            {
                rtcSeconds++;

                SET_BIT(PORTB, 5);    // Turn heartbeat LED ON
                LEDturnOffTime = milliseconds() + 100;

                if (rtcSeconds >= 60)  // 60 seconds up; "roll-over" minute
                {
                    rtcSeconds = 0;
                    rtcMinutes++;
                }
                if	(rtcMinutes >= 60)  // 60 minutes up; "roll-over" Hour
                {
                    rtcMinutes = 0;
                    rtcHour++;
                }
                if	(rtcHour >= 24)  // 24 hours up; "roll-over" day
                {
                    rtcHour = 0;
                }
            }
        }

        if (isTaskPending_50ms())
		{
			ButtonScan(4);       // ButtonScan is called every 50ms
			DisplayTimeOfDay();  // update display every 50ms
		}

        // Check if the LED on-time (duty) has reached 100ms...
        if (milliseconds() >= LEDturnOffTime)  CLEAR_BIT(PORTB, 5);  // Turn LED off

        // test for button A held down --> setting mode
        if (button_pressed('A'))  SettingTimeOfDay();

		if (button_hit('D'))  display_mode = !display_mode;  // toggle the mode
    }
}

/*
 * Convert rtcHour, rtcMinutes and rtcSeconds to printable digits.
 * Each value has a maximum of 2 non-zero digits. Show leading zeros.
 * Using LCD library functions, display the 6 digits hh, mm, ss
 * in their correct positions on the bottom row of the LCD.
 * Insert a colon between hh and mm, also between mm and ss.
 * (Optional) Suppress (i.e. make blank) leading zero on hours (only).
 *
 */
void  DisplayTimeOfDay()
{
    BYTE  hours_12;

    // Set hours_12 for 12-hour AM/PM display mode...
    if (rtcHour == 0)  hours_12 = 12;  // = 12:xx AM
	else if (rtcHour > 12)  hours_12 = rtcHour - 12;  // 1pm onwards
	else  hours_12 = rtcHour;

    lcd_cursor_posn(1, 2);

	if (display_mode != 0)   // 12-hour mode selected
	{
		if ((hours_12 / 10) == 0) lcd_write_char(' ');
		else  lcd_write_char((hours_12 / 10) + '0');
		lcd_write_char((hours_12 % 10) +'0');
	}
	else   // 24-hour mode selected
	{
		if ((rtcHour / 10) == 0) lcd_write_char(' ');
		else  lcd_write_char((rtcHour / 10) + '0');
		lcd_write_char((rtcHour % 10) +'0');
	}

	lcd_print_string(":");
    lcd_write_char((rtcMinutes / 10) +'0' );
    lcd_write_char((rtcMinutes % 10) +'0' );
    lcd_print_string(":");

    lcd_write_char((rtcSeconds / 10) +'0' );
    lcd_write_char((rtcSeconds % 10) +'0' );

	if (display_mode != 0)   // 12-hour mode selected
	{
		lcd_write_char(' ');
		if (rtcHour < 12)  lcd_print_string("am");
		else  lcd_print_string("pm");
	}
	else  // 24-hour mode -- clear the am/pm indicator
	{
		lcd_cursor_posn(1, 10);
		lcd_print_string("    ");
	}
}


/*
 * To do: Describe workings of this function.  (What does it do and how?)
 * ``````````````````````````````````````````````````````````````````````
 */
void  SettingTimeOfDay()
{
    // if button B is hit, adjust hours setting
    if ( button_hit('B') )
	{
		rtcHour++;
		if (rtcHour >= 24) rtcHour = 0;
	}

    // if button C is hit, adjust minutes setting
	if ( button_hit('C') )
	{
		rtcMinutes++;
		if (rtcMinutes >= 60) rtcMinutes = 0;
	}

    rtcSeconds = 0;
}

// end of file